//
// This class loads a csv file and displays it on the screen as a table.
//
// Cells that can be parsed as an int or as a float are displayed in blue.
// Use mouse drag to pan the table.
//
// @author Pierre Dragicevic, updated for Processing 2 by James Eagan.
//

class TableViewer {
  
  int maxColSize = 20; // max column width in characters
  Table table;
  PFont f;
  int w, h, rows, cols;
  int[] colSizes = null;
  boolean[][] isNumeric = null;
  
  // Creates a new table viewer.
  //
  // If parseNumericalData is true, will try to parse numerical data and show it in blue when successful. Warning: can take
  // forever on big datasets. If showParsedNumericalData is true, will show numeric data after parsing, i.e., correctly formatted.
  TableViewer(Table table, boolean parseNumericalData, boolean showParsedNumericalData) {
  
    this.table = table;
    rows = table.getRowCount();
    cols = table.getColumnCount();
     
    // Test and parse numeric values.
    isNumeric = new boolean[cols][rows];
    if (parseNumericalData) {
      print("Parsing numerical data... ");
      for (int c=0; c<cols; c++) {
        for (int r=0; r<rows; r++) {
          int intValue = table.smartParseInt(table.get(c, r));
          if (intValue == Integer.MAX_VALUE) {
            float floatValue = table.smartParseFloat(table.get(c, r));
            if (Float.isNaN(floatValue)) {
              isNumeric[c][r] = false;
            } else {
              if (showParsedNumericalData)
                table.set(c, r, "" + floatValue);
              isNumeric[c][r] = true;
            }
          } else {
            if (showParsedNumericalData)
              table.set(c, r, "" + intValue);
            isNumeric[c][r] = true;
          }
        }
      }
      println("Done.");
    }
    
    // Compute column sizes.
    colSizes = new int[cols];
    for (int c=0; c<cols; c++) {
      colSizes[c] = table.getColumnName(c).length();
      for (int r=0; r<rows; r++) {
        int l = table.get(c, r).length();
        if (l > colSizes[c])
          colSizes[c] = l;
      }
      colSizes[c] = min(colSizes[c], maxColSize);
    }
      
    // This font has been created using the Tools menu in the Processing IDE.
    f = loadFont("CourierNewPSMT-11.vlw");
    
    w = (int)(displayWidth * 0.7);
    h = (int)(displayHeight * 0.8);
    size(w, h);
  }
  
  int x0 = 10;
  int y0 = 20;
  
  void draw() {
    
    background(255);
       
    // Draw the table
    textFont(f);
    fill(0);
    int xspace = 7;
    int yspace = 10;
   
    int x = x0;
    for (int c=0; c<cols; c++) {
        // Draw background
        if (c%2 == 1) {
          fill(240, 240, 240);
          noStroke();
          rect(x - 5, 0, colSizes[c] * xspace + 10, h);
        }
        // Draw column values
        fill(0);
        int firstRow = max(0, -y0 / yspace - 2);
        int lastRow = min(rows-1, (-y0 + h) / yspace + 2);
        for (int r=firstRow; r<=lastRow; r++) {
          String s = table.get(c, r);
          if (isNumeric[c][r])
            fill(50, 50, 255);
          else
            fill(50, 50, 50);
          if (s.length() > maxColSize)
            s = s.substring(0, maxColSize) + "…";
          text(s, x, y0 + (r + 1) * yspace + 10);
        }
        // Draw column name
        fill(255);
        rect(x - 5, 0, colSizes[c] * xspace + 15, yspace*2+5);
        String s = table.getColumnName(c);
        if (s.length() > maxColSize)
          s = s.substring(0, maxColSize) + "…";
        fill(0);
        text(s, x, 15);
        x += colSizes[c] * xspace + 10;
    }
  }
  
  int mx, my;
  
  void mousePressed() {
    mx = mouseX;
    my = mouseY;
  }
  
  void mouseDragged() {
    x0 += mouseX - mx;
    y0 += (mouseY - my)*10;
    if (x0 > 10)
      x0 = 10;
    if (y0 > 20)
      y0 = 20;
    mx = mouseX;
    my = mouseY;
    redraw();
  }
}
